"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const foreign_key_service_1 = require("../foreign-key/foreign-key-service");
const model_not_initialized_error_1 = require("../../model/shared/model-not-initialized-error");
class BaseAssociation {
    constructor(associatedClassGetter, options) {
        this.associatedClassGetter = associatedClassGetter;
        this.options = options;
    }
    getAssociatedClass() {
        const modelClass = this.associatedClassGetter();
        if (!modelClass.isInitialized) {
            throw new model_not_initialized_error_1.ModelNotInitializedError(modelClass, {
                cause: 'before association can be resolved.'
            });
        }
        return modelClass;
    }
    getAs() {
        return this.options.as;
    }
    getForeignKeyOptions(relatedClass, classWithForeignKey, foreignKey) {
        let foreignKeyOptions = {};
        if (typeof foreignKey === 'string') {
            foreignKeyOptions.name = foreignKey;
        }
        else if (foreignKey && typeof foreignKey === 'object') {
            foreignKeyOptions = Object.assign({}, foreignKey);
        }
        if (!foreignKeyOptions.name) {
            const foreignKeys = foreign_key_service_1.getForeignKeys(classWithForeignKey.prototype) || [];
            for (const key of foreignKeys) {
                if (key.relatedClassGetter() === relatedClass) {
                    foreignKeyOptions.name = key.foreignKey;
                    break;
                }
            }
            if (!foreignKeyOptions.name) {
                throw new Error(`Foreign key for "${relatedClass.name}" is missing ` +
                    `on "${classWithForeignKey.name}".`);
            }
        }
        return foreignKeyOptions;
    }
}
exports.BaseAssociation = BaseAssociation;
//# sourceMappingURL=base-association.js.map